@@ -71,6 +71,8 @@ struct ovs_chassis_cfg {
     bool is_interconn;
     /* Does OVS support sampling with ids taken from registers? */
     bool sample_with_regs;
+    /* Does OVS support flushing CT zones using label/mark? */
+    bool ct_label_flush;
 };
 
 static void
@@ -243,12 +245,16 @@ chassis_parse_ovs_encap_type(const char *encap_type,
 {
     sset_from_delimited_string(encap_type_set, encap_type, ",");
 
+    static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
     const char *type;
 
     SSET_FOR_EACH (type, encap_type_set) {
-        if (!get_tunnel_type(type)) {
-            static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
+        uint32_t tun_type = get_tunnel_type(type);
+
+        if (!tun_type) {
             VLOG_INFO_RL(&rl, "Unknown tunnel type: %s", type);
+        } else if (tun_type == STT) {
+            VLOG_WARN_RL(&rl, "STT encapsulation type is deprecated");
         }
     }
 }
@@ -354,6 +360,8 @@ chassis_parse_ovs_config(const struct ovsrec_open_vswitch_table *ovs_table,
     ovs_cfg->is_interconn = get_is_interconn(&cfg->external_ids, chassis_id);
     ovs_cfg->sample_with_regs =
         ovs_feature_is_supported(OVS_SAMPLE_REG_SUPPORT);
+    ovs_cfg->ct_label_flush =
+        ovs_feature_is_supported(OVS_CT_LABEL_FLUSH_SUPPORT);
 
     return true;
 }
@@ -390,6 +398,9 @@ chassis_build_other_config(const struct ovs_chassis_cfg *ovs_cfg,
     smap_replace(config, OVN_FEATURE_CT_COMMIT_TO_ZONE, "true");
     smap_replace(config, OVN_FEATURE_SAMPLE_WITH_REGISTERS,
                  ovs_cfg->sample_with_regs ? "true" : "false");
+    smap_replace(config, OVN_FEATURE_CT_NEXT_ZONE, "true");
+    smap_replace(config, OVN_FEATURE_CT_LABEL_FLUSH,
+                 ovs_cfg->ct_label_flush ? "true" :"false");
 }
 
 /*
@@ -549,6 +560,20 @@ chassis_other_config_changed(const struct ovs_chassis_cfg *ovs_cfg,
         return true;
     }
 
+    if (!smap_get_bool(&chassis_rec->other_config,
+                       OVN_FEATURE_CT_NEXT_ZONE,
+                       false)) {
+        return true;
+    }
+
+    bool chassis_ct_label_flush =
+        smap_get_bool(&chassis_rec->other_config,
+                      OVN_FEATURE_CT_LABEL_FLUSH,
+                      false);
+    if (chassis_ct_label_flush != ovs_cfg->ct_label_flush) {
+        return true;
+    }
+
     return false;
 }
 
@@ -706,6 +731,8 @@ update_supported_sset(struct sset *supported)
     sset_add(supported, OVN_FEATURE_CT_COMMIT_NAT_V2);
     sset_add(supported, OVN_FEATURE_CT_COMMIT_TO_ZONE);
     sset_add(supported, OVN_FEATURE_SAMPLE_WITH_REGISTERS);
+    sset_add(supported, OVN_FEATURE_CT_NEXT_ZONE);
+    sset_add(supported, OVN_FEATURE_CT_LABEL_FLUSH);
 }
 
 static void
@@ -1016,6 +1043,9 @@ store_chassis_index_if_needed(
 {
     const struct ovsrec_open_vswitch *cfg =
         ovsrec_open_vswitch_table_first(ovs_table);
+    if (!cfg) {
+        return;
+    }
     const char *chassis_id = get_ovs_chassis_id(ovs_table);
 
     char *idx_key = xasprintf(CHASSIS_IDX_PREFIX "%s", chassis_id);
