@@ -131,6 +131,7 @@ struct tracked_datapath *tracked_datapath_find(
 void tracked_datapath_lport_add(const struct sbrec_port_binding *,
                                 enum en_tracked_resource_type,
                                 struct hmap *tracked_datapaths);
+void tracked_datapaths_clear(struct hmap *tracked_datapaths);
 void tracked_datapaths_destroy(struct hmap *tracked_datapaths);
 
 /* Maps from a chassis to the OpenFlow port number of the tunnel that can be
@@ -173,5 +174,7 @@ void remove_local_datapath_multichassis_port(struct local_datapath *ld,
                                              char *logical_port);
 bool lb_is_local(const struct sbrec_load_balancer *sbrec_lb,
                  const struct hmap *local_datapaths);
+void local_data_dump_peer_ports(struct hmap *local_datapaths,
+                                struct ds *peer_ports);
 
 #endif /* controller/local_data.h */
