@@ -48,10 +48,13 @@ struct mac_cache_threshold {
     uint64_t value;
     /* Statistics dump period. */
     uint64_t dump_period;
+    /* How long to wait between two database updates. */
+    uint64_t cooldown_period;
 };
 
 struct mac_binding_data {
     /* Keys. */
+    uint64_t cookie;
     uint32_t port_key;
     uint32_t dp_key;
     struct in6_addr ip;
@@ -64,7 +67,7 @@ struct mac_binding {
     /* Common data to identify MAC binding. */
     struct mac_binding_data data;
     /* Reference to the SB MAC binding record (Might be NULL). */
-    const struct sbrec_mac_binding *sbrec_mb;
+    const struct sbrec_mac_binding *sbrec;
     /* User specified timestamp (in ms) */
     long long timestamp;
 };
@@ -128,26 +131,37 @@ void mac_cache_thresholds_sync(struct mac_cache_data *data,
 void mac_cache_thresholds_clear(struct mac_cache_data *data);
 
 /* MAC binding. */
-struct mac_binding *mac_binding_add(struct hmap *map,
-                                    struct mac_binding_data mb_data,
-                                    long long timestamp);
+
+static inline void
+mac_binding_data_init(struct mac_binding_data *data,
+                      uint32_t dp_key, uint32_t port_key,
+                      struct in6_addr ip, struct eth_addr mac)
+{
+    *data = (struct mac_binding_data) {
+        .cookie = 0,
+        .dp_key = (dp_key),
+        .port_key = (port_key),
+        .ip = (ip),
+        .mac = (mac),
+    };
+}
+
+void mac_binding_add(struct hmap *map, struct mac_binding_data mb_data,
+                     const struct sbrec_mac_binding *, long long timestamp);
 
 void mac_binding_remove(struct hmap *map, struct mac_binding *mb);
 
 struct mac_binding *mac_binding_find(const struct hmap *map,
                                      const struct mac_binding_data *mb_data);
 
+bool mac_binding_data_parse(struct mac_binding_data *data,
+                            uint32_t dp_key, uint32_t port_key,
+                            const char *ip_str, const char *mac_str);
 bool mac_binding_data_from_sbrec(struct mac_binding_data *data,
-                                 const struct sbrec_mac_binding *mb,
-                                 struct ovsdb_idl_index *sbrec_pb_by_name);
+                                 const struct sbrec_mac_binding *mb);
 
 void mac_bindings_clear(struct hmap *map);
-
-bool sb_mac_binding_updated(const struct sbrec_mac_binding *mb);
-
-const struct sbrec_mac_binding *
-mac_binding_lookup(struct ovsdb_idl_index *sbrec_mac_binding_by_lport_ip,
-                   const char *logical_port, const char *ip);
+void mac_bindings_to_string(const struct hmap *map, struct ds *out_data);
 
 /* FDB. */
 struct fdb *fdb_add(struct hmap *map, struct fdb_data fdb_data,
@@ -159,8 +173,6 @@ bool fdb_data_from_sbrec(struct fdb_data *data, const struct sbrec_fdb *fdb);
 
 struct fdb *fdb_find(const struct hmap *map, const struct fdb_data *fdb_data);
 
-bool sb_fdb_updated(const struct sbrec_fdb *fdb);
-
 void fdbs_clear(struct hmap *map);
 
 /* MAC binding stat processing. */
@@ -168,15 +180,19 @@ void
 mac_binding_stats_process_flow_stats(struct ovs_list *stats_list,
                                      struct ofputil_flow_stats *ofp_stats);
 
-void mac_binding_stats_run(struct ovs_list *stats_list, uint64_t *req_delay,
-                           void *data);
+void mac_binding_stats_run(
+        struct rconn *swconn OVS_UNUSED,
+        struct ovsdb_idl_index *sbrec_port_binding_by_name OVS_UNUSED,
+        struct ovs_list *stats_list, uint64_t *req_delay, void *data);
 
 /* FDB stat processing. */
 void fdb_stats_process_flow_stats(struct ovs_list *stats_list,
                                   struct ofputil_flow_stats *ofp_stats);
 
-void fdb_stats_run(struct ovs_list *stats_list, uint64_t *req_delay,
-                   void *data);
+void fdb_stats_run(
+        struct rconn *swconn OVS_UNUSED,
+        struct ovsdb_idl_index *sbrec_port_binding_by_name OVS_UNUSED,
+        struct ovs_list *stats_list, uint64_t *req_delay, void *data);
 
 void mac_cache_stats_destroy(struct ovs_list *stats_list);
 
@@ -209,4 +225,13 @@ bool buffered_packets_ctx_is_ready_to_send(struct buffered_packets_ctx *ctx);
 
 bool buffered_packets_ctx_has_packets(struct buffered_packets_ctx *ctx);
 
+void mac_binding_probe_stats_process_flow_stats(
+        struct ovs_list *stats_list,
+        struct ofputil_flow_stats *ofp_stats);
+
+void mac_binding_probe_stats_run(
+        struct rconn *swconn,
+        struct ovsdb_idl_index *sbrec_port_binding_by_name,
+        struct ovs_list *stats_list, uint64_t *req_delay, void *data);
+
 #endif /* controller/mac-cache.h */
