@@ -30,6 +30,8 @@
 #define OVN_FEATURE_CT_COMMIT_NAT_V2 "ct-commit-nat-v2"
 #define OVN_FEATURE_CT_COMMIT_TO_ZONE "ct-commit-to-zone"
 #define OVN_FEATURE_SAMPLE_WITH_REGISTERS "ovn-sample-with-registers"
+#define OVN_FEATURE_CT_NEXT_ZONE "ct-next-zone"
+#define OVN_FEATURE_CT_LABEL_FLUSH "ct-label-flush"
 
 /* OVS datapath supported features.  Based on availability OVN might generate
  * different types of openflows.
@@ -41,6 +43,7 @@ enum ovs_feature_support_bits {
     OVS_DP_HASH_L4_SYM_BIT,
     OVS_OF_GROUP_SUPPORT_BIT,
     OVS_SAMPLE_REG_SUPPORT_BIT,
+    OVS_CT_LABEL_FLUSH_BIT,
 };
 
 enum ovs_feature_value {
@@ -50,14 +53,17 @@ enum ovs_feature_value {
     OVS_DP_HASH_L4_SYM_SUPPORT = (1 << OVS_DP_HASH_L4_SYM_BIT),
     OVS_OF_GROUP_SUPPORT = (1 << OVS_OF_GROUP_SUPPORT_BIT),
     OVS_SAMPLE_REG_SUPPORT = (1 << OVS_SAMPLE_REG_SUPPORT_BIT),
+    OVS_CT_LABEL_FLUSH_SUPPORT = (1 << OVS_CT_LABEL_FLUSH_BIT),
 };
 
 void ovs_feature_support_destroy(void);
 bool ovs_feature_is_supported(enum ovs_feature_value feature);
 bool ovs_feature_support_run(const struct smap *ovs_capabilities,
-                             const char *conn_target, int probe_interval);
+                             const char *conn_target, int probe_interval,
+                             const char *db_target);
 bool ovs_feature_set_discovered(void);
 uint32_t ovs_feature_max_meters_get(void);
 uint32_t ovs_feature_max_select_groups_get(void);
+size_t ovs_features_max_flow_table_prefixes_get(void);
 
 #endif
