@@ -22,5 +22,8 @@ bool lflow_northd_handler(struct engine_node *, void *data);
 bool lflow_port_group_handler(struct engine_node *, void *data);
 bool lflow_lr_stateful_handler(struct engine_node *, void *data);
 bool lflow_ls_stateful_handler(struct engine_node *node, void *data);
+bool lflow_multicast_igmp_handler(struct engine_node *node, void *data);
+bool lflow_group_ecmp_route_change_handler(struct engine_node *node,
+                                           void *data);
 
 #endif /* EN_LFLOW_H */
