@@ -17,5 +17,11 @@ bool northd_output_mac_binding_aging_handler(struct engine_node *node,
                                              void *data OVS_UNUSED);
 bool northd_output_fdb_aging_handler(struct engine_node *node,
                                      void *data OVS_UNUSED);
+bool northd_output_ecmp_nexthop_handler(struct engine_node *node,
+                                        void *data OVS_UNUSED);
+bool northd_output_acl_id_handler(struct engine_node *node,
+                                  void *data OVS_UNUSED);
+bool northd_output_advertised_route_sync_handler(struct engine_node *node,
+                                                 void *data OVS_UNUSED);
 
 #endif
